# -*- coding:utf-8 -*-
# CREATED BY: bohuai jiang 
# CREATED ON: 2020/6/8 4:36 PM
# LAST MODIFIED ON:
# AIM: 测试模型速度

from sentence_spliter.sentence_spliter import Spliter
import matplotlib.pyplot as plt
import time
from utility.time_it import time_measure


def check(options, sentence_list) -> bool:
    for sentence in sentence_list:
        if not (options['min_length'] <= len(sentence) <= options['max_length']):
            return False
    else:
        return True


def run_single(options, sample_size=1):
    sentence = '(太阳当空照).(花儿对我笑)，"小鸟说早早"，"你为什么背上书包了,".　\
    我去上学校,天天不迟到，爱学习爱劳.' * sample_size
    spliter = Spliter(options)
    start_t = time.time()
    cut_sentences = spliter.cut_to_sentences(sentence)
    end_t = time.time() - start_t
    return end_t, cut_sentences


def test(options):
    time = []
    size = []
    for i in range(1000, 1000000, 1000):
        t, out = run_single(options, i)
        if not check(options, sentence_list=out):
            raise Exception('分句错误')
        time.append(t)
        size.append(i)
        if i % 100 == 0:
            print(i)
            plt.figure()
            plt.plot(time)
            plt.savefig('perform')
            plt.close()


if __name__ == '__main__':
    options = {
        'language': 'zh',  # 'zh'中文 'en' 英文
        'long_short_sent_handle': 'y',  # 'y'自然切分，不处理长短句；'n'处理长短句
        'max_length': 5,  # 最长句子，默认值150
        'min_length': 4,  # 最短句子，默认值15
        'hard_max_length': 5,  # 强制截断，默认值300
        'remove_blank': True
    }
    test(options)
    #print(run_single(options, 15000))



# -*- coding:utf-8 -*-
# CREATED BY: bohuai jiang 
# CREATED ON: 2020/8/7 3:54 PM
# LAST MODIFIED ON:
# AIM:
from abc import ABC

from sentence_spliter.automata.abc import Operation
from sentence_spliter.automata.sequence import StrSequence
from sentence_spliter.automata import condition
import copy


class Indolent(Operation):
    def __init__(self, name: str = None):
        super(Indolent, self).__init__('Indolent' if not name else name)

    def operate(self, state: StrSequence) -> StrSequence:
        state.add_to_candidate()
        return state


class Normal(Operation):
    def __init__(self):
        super(Normal, self).__init__('Normal')

    def operate(self, state: StrSequence) -> StrSequence:
        state.add_to_sentence_list()
        return state


class LongHandler(Operation):
    def __init__(self, hard_max: int = 300):
        super(LongHandler, self).__init__('LongHandler')
        self.is_end_symbol = condition.IsEndSymbol()
        self.is_comma = condition.IsComma()
        self.is_book_close = condition.IsBookClose()
        self.is_barcket_close = condition.IsBracketClose()
        self.is_right_quota = condition.IsRightQuota()
        self.hard_max = hard_max

    def operate(self, state: StrSequence) -> StrSequence:
        # new_candidate = []
        # - step 1. end_symbol 过滤
        temps_state = copy.copy(state)
        for i in range(temps_state.sentence_start, temps_state.v_pointer - 1)[::-1]:
            temps_state.v_pointer = i
            if self.is_right_quota(temps_state) or (self.is_end_symbol(temps_state) and self.is_book_close(temps_state)):
                cut_id = i
                break
        else:
            temps_state = copy.copy(state)
            # - step 2. comma 过滤
            for i in range(temps_state.sentence_start, temps_state.v_pointer - 1)[::-1]:  # enumerate(state.candidate):
                temps_state.v_pointer = i
                if self.is_comma(temps_state):
                    cut_id = i
                    break
            else:
                cut_id = state.sentence_start + self.hard_max - 1

        state.v_pointer = cut_id

        return state


class ShortHandler(Operation):

    def __init__(self):
        super(ShortHandler, self).__init__('ShortHandler')

    def operate(self, state: StrSequence) -> StrSequence:
        state.add_to_candidate()
        return state


class EndState(Operation):
    def __init__(self):
        super(EndState, self).__init__('EndState')

    def operate(self, state: StrSequence) -> StrSequence:
        if state.sentence_list and state.sentence_start >= state.length or state.length == 0:
            return state
        else:
            state.add_to_sentence_list()
            return state


# =========================== #
#      Special Condition      #
# =========================== #

class SpecialOperation(Operation, ABC):
    pass



from setuptools import setup, find_packages

setup(
    name="sentence_spliter",
    version="1.0.9",
    author="bohuai jiang",
    author_email="highjump000@hotmail.com",
    description="This is a sentence cutting tool that supports long sentence segmentation and short sentence merging.",
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    license="MIT",
    # package_dir={"":"sentence_spliter"},
    packages=['sentence_spliter'],

    classifiers=[
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "License :: OSI Approved :: MIT License",
    ],
    REQUIRES_PYTHON='>=3.6.0',
    install_requires=['attrdict==2.0.1']
)
